<?php 

/**
 * Generate U-Design theme related CMB2 metaboxes, custom fields, or forms
 * 
 */
if ( !defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


/**
 * Get the cmb2 bootstrap!
 */
if ( file_exists( dirname( __FILE__ ) . '/cmb2/init.php' ) ) {
    require_once dirname( __FILE__ ) . '/cmb2/init.php';
} elseif ( file_exists( dirname( __FILE__ ) . '/CMB2/init.php' ) ) {
    require_once dirname( __FILE__ ) . '/CMB2/init.php';
}


// Include the CMB2 RGBa Colorpicker (3rd Party Resource)
if ( file_exists( dirname( __FILE__ ) . '/cmb2/3rd-party-resources/CMB2_RGBa_Picker-master/jw-cmb2-rgba-colorpicker.php' ) ) {
    require_once dirname( __FILE__ ) . '/cmb2/3rd-party-resources/CMB2_RGBa_Picker-master/jw-cmb2-rgba-colorpicker.php';
}



add_action( 'cmb2_init', 'udesign_cmb2_metaboxes' );
/**
 * Define the metabox and field configurations.
 */
function udesign_cmb2_metaboxes() {
    
    
    // Start with an underscore to hide fields from custom fields list
    $prefix = '_udesign_';

    /**
     * Initiate the metabox
     */
    $cmb = new_cmb2_box( array(
        'id'            => 'udesign_metabox',
        'title'         => __( 'U-Design Options', 'udesign' ),
        'object_types'  => array( 'post', 'page' ), // Post type
        'context'       => 'normal',
        'priority'      => 'high',
        'show_names'    => true, // Show field names on the left
        'closed'        => false, // Keep the metabox closed by default
    ) );

    
    /*******************************( PAGE TITLE )***********************************/
    
    // Page Title select options
    $cmb->add_field( array(
        'name'              => __( 'Title Position', 'udesign' ),
        'desc'              => __( 'Overwrite the global title position (applies to this page only)', 'udesign' ),
        'id'                => $prefix . 'page_title',
        'type'              => 'select',
        'show_option_none'  => false,
        'default'           => 'default_position',
        'options'           => array(
            'default_position'   => __( 'Use the default position', 'udesign' ),
            'position1'         => __( 'Title Position 1 (before the main content)', 'udesign' ),
            'position2'         => __( 'Title Position 2 (inside the main content)', 'udesign' ),
            'remove1'           => __( 'Remove Title (SEO-friendly)', 'udesign' ),
            'remove2'           => __( 'Remove Title Completely', 'udesign' ),
        ),
    ) );
    
    
    
    /*******************************( BREADCRUMBS )***********************************/
    
    // Breadcrumbs option checkbox
    $cmb->add_field( array(
        'name' => __( 'Breadcrumbs', 'udesign' ),
        //'desc' => __( 'The description (optional)', 'udesign' ),
        'id'   => $prefix . 'page_breadcrumbs_options',
        'type' => 'multicheck',
        'select_all_button' => false,
        'default' => 'cmb2_set_checkboxes_default_for_breadcrumbs_options', // use a call back function to determine whether the deprecated value exists and update the new one accordingly
        'options' => array(
            'disable_breadcrumbs' => sprintf( esc_html__('Disable Breadcrumbs - %1$sThis option will disable/hide the breadcrumbs on this page only%2$s', 'udesign'), '<span class="cmb2-metabox-description">', '</span>'),
            'move_to_title' => sprintf( esc_html__('Move Breadcrumbs to title area - %1$sThis only applies for "Title Position 1" %2$s', 'udesign'), '<span class="cmb2-metabox-description">', '</span>'),
        )
    ) );
    
    /**
     * backward compatability: check if the user had disabled breadcrumbs for this particular post
     * with the old single checkbox option '_udesign_disable_breadcrumbs' option
     */
    function cmb2_set_checkboxes_default_for_breadcrumbs_options() {
        $defaults =  array();
        if ( get_post_meta( get_the_ID(), '_udesign_disable_breadcrumbs', true ) ) {
            $defaults =  array('disable_breadcrumbs');
            update_post_meta( get_the_ID(), '_udesign_page_breadcrumbs_options', $defaults ); // assign the deprecated value to the new
            delete_post_meta( get_the_ID(), '_udesign_disable_breadcrumbs' ); // delete the deprecated post meta option
        }
        return $defaults;
    }

    
    
    /*******************************( PAGE LAYOUT )***********************************/
    
    // Page layout options
    $cmb->add_field( array(
        'name'    => __( 'Page Layout Options', 'udesign' ),
        'desc'    => __( 'Select the sections above you would like to remove from this page', 'udesign' ),
        'id'      => $prefix . 'page_layout_options',
        'type'    => 'multicheck',
        'select_all_button' => false,
        'options' => 'cmb_get_the_page_layout_options_array',
    ) );
    
    /**
     * conditionally build the multicheck options based on which page template is used
     */
    function cmb_get_the_page_layout_options_array() {
        $options =  array(
            'no_header'     => sprintf( esc_html__('No Header - %1$s Remove all top area items (main menu, logo, secondary menu bar, etc.)%2$s', 'udesign'), '<span class="cmb2-metabox-description">', '</span>'),
            'no_container'  => sprintf( esc_html__('No Container - %1$s Remove main content\'s inner wrapping divs%2$s', 'udesign'), '<span class="cmb2-metabox-description">', '</span>'),
            'no_bottom'     => __( 'No Bottom area', 'udesign' ),
            'no_footer'     => __( 'No Footer', 'udesign' ),
        );
        if ( get_page_template_slug( get_the_ID() ) !== 'page-FullWidth.php' ) {
            unset( $options['no_container'] );
        }
        return $options;
    }
    
    
    
    /*******************************( CUSTOM HEADER )***********************************/
    // get the image
    $cmb->add_field( array(
        'name'      => __( 'Header Image', 'udesign' ),
        'desc'      => __( 'Upload an image or enter a URL', 'udesign' ),
        'id'        => $prefix . 'page_header_image',
        'type'      => 'file',
        // Optional:
        'options'   => array(
            'url'   => false, // Show the text input for the url
            'add_upload_file_text' => __( 'Add Image', 'udesign' ) // Upload button text. Default: "Add or Upload File"
        ),
        'after_row' => 'insert_custom_styles', // use this callback to isert some custom CSS and JS to style the CUSTOM HEADER options as one group
    ) );
    
    $cmb->add_field( array(
        'name'      => __( 'Top Area Over Header Image', 'udesign' ),
        'desc'      => __( 'Position the top area over the header image', 'udesign' ),
        'id'        => $prefix . 'page_top_area_over_header_image',
        'type'      => 'checkbox',
	'before_field' => sprintf( esc_html__('%1$sTop Area Over Header Image:%2$s', 'udesign'), '<p><strong>', '</strong></p>'),
	'before_row' => '<div id="top-area-over-header-optns-wrapper">',
    ) );
    
    // top area color
    $cmb->add_field( array(
        'name'      => __( 'Top Area Background Color', 'udesign' ),
        'desc'      => __( 'Pick a color for the top area background which will go over the image, ', 'udesign' ),
        'id'        => $prefix . 'page_top_area_bg',
        'type'      => 'rgba_colorpicker',
        'default'   => 'rgba(0,0,0,0.1)',
	'after_row' => '</div>', // closing div for #top-area-over-header-optns-wrapper
    ) );
    
    function insert_custom_styles( $args, $field ) {
        ob_start(); ?>
            <style>
                .cmb2-id--udesign-page-header-image,
                .cmb2-id--udesign-page-top-area-over-header-image { border-bottom: none !important; }
                .cmb-th label[for="_udesign_page_top_area_over_header_image"],
                .cmb-th label[for="_udesign_page_top_area_bg"] {
                    display: none !important;
                }
                .cmb-row.cmb2-id--udesign-page-top-area-bg, #top-area-over-header-optns-wrapper { display: none; }
                .cmb-row.cmb2-id--udesign-page-top-area-bg, .cmb-row.cmb2-id--udesign-page-top-area-over-header-image { margin-top: -20px; }
            </style>
            <script type="text/javascript">
                // <![CDATA[
                jQuery(document).ready(function($){
                    
                    // on page load: determine whether an image has been selected and show/hide dependent options accordingly
                    if( $("#_udesign_page_header_image_id-status .img-status").length == 1 ) { // image selected
                         $("#top-area-over-header-optns-wrapper").fadeIn( 150 );
                    }
                    // when image is selected or deleted: show/hide dependent options accordingly
                    var container = document.querySelector('#_udesign_page_header_image_id-status');
                    if (typeof MutationObserver === 'function') { // exclude browsers that don't support "MutationObserver"
                        var observer = new MutationObserver(function(mutations){
                            if( $("#_udesign_page_header_image_id-status .img-status").length == 0 ) { // image removed
                                 $("#top-area-over-header-optns-wrapper").fadeOut( 250 );
                            } else {  // image selected
                                 $("#top-area-over-header-optns-wrapper").fadeIn( 450 );
                            }
                        });
                        observer.observe(container, {childList: true}); // we just need to monitor for changes to the "container" element's children
                    }
                
                    // on page load: show the color picker if the checkbox is checked
                    if ( $("input#_udesign_page_top_area_over_header_image").is(':checked')  ) {
                        $(".cmb-row.cmb2-id--udesign-page-top-area-bg").fadeIn( 250 );
                    }
                    // toggle visibility for the color picker when the checkbox is toggled
                    $( "input#_udesign_page_top_area_over_header_image" ).click(function() {
                        if ( $("input#_udesign_page_top_area_over_header_image").is(':checked')  ) {
                            $(".cmb-row.cmb2-id--udesign-page-top-area-bg").fadeIn( 250 );
                        } else {
                            $(".cmb-row.cmb2-id--udesign-page-top-area-bg").fadeOut( 350 );
                        }
                    });
                });
                // ]]>
            </script>
        <?php
        return ob_get_clean();
    }
    
}








